/*
    File:       BRDF.h

    Function:	

    Author:     Andrew Willmott

    Copyright:  (c) 1999-2000, Andrew Willmott
*/

#ifndef __BRDF__
#define __BRDF__

#include "Colour.h"
#include "Geometry.h"

class BRDFMtl
{
public:
	virtual Colour	BRDF(const Vector &normal, 
						const Vector &light, const Vector &view) = 0;
};


class Phong : public BRDFMtl
{
public:
	Phong(GCLReal rd, GCLReal rs, GCLReal rn, Colour rdClr, Colour rsClr);
	
	Colour	BRDF(const Vector &normal, 
					const Vector &light, const Vector &view);

	GCLReal		rd, rs, rn;
	Colour		rdClr;	
	Colour		rsClr;	
};

class PhongH : public BRDFMtl
{
public:
	PhongH(GCLReal rd, GCLReal rs, GCLReal rn, Colour rdClr, Colour rsClr);
	
	Colour	BRDF(const Vector &normal, 
					const Vector &light, const Vector &view);

	GCLReal		rd, rs, rn;
	Colour		rdClr;	
	Colour		rsClr;	
};

class CookTorrance : public BRDFMtl
{
public:
	CookTorrance(GCLReal m, GCLReal d, GCLReal n, Colour rdClr);
	
	Colour		BRDF(const Vector &N, const Vector &L, const Vector &V);

	GCLReal		m;
	GCLReal		d;
	GCLReal		n;
	Colour		rdClr;
};

class HeBRDF : public BRDFMtl
{
public:
	HeBRDF(GCLReal tau, GCLReal sigma0, GCLReal d, GCLReal n_r, GCLReal n_i);
	
	Colour		BRDF(const Vector &N, const Vector &L, const Vector &V);

	GCLReal			shadowf(GCLReal t);
	GCLReal			kK(GCLReal t);
	GCLReal			gFunc(GCLReal ti, GCLReal tr);
	GCLReal			kD(const Vector &ki, const Vector &kr, const Vector &ns);
	GCLReal			kG(const Vector &ki, const Vector &kr, const Vector &ns);
	GCLReal			z0d(GCLReal Ki, GCLReal z);
	GCLReal			z0lim(GCLReal x);
	GCLReal			z0off(GCLReal Ki, GCLReal z);
	GCLReal			z0rootf(GCLReal Ki, GCLReal x0, GCLReal x1, Int i);
	GCLReal			z0root(GCLReal ti, GCLReal tr);
	GCLReal			sigma(GCLReal ti, GCLReal tr);

	GCLReal			rhoSp(const Vector &ki, const Vector &kr, const Vector &ns);
	GCLReal			rhoDD(const Vector &ki, const Vector &kr, const Vector &ns);
	GCLReal			rhoUD(const Vector &ki, const Vector &kr, const Vector &ns);

	GCLReal		tau;
	GCLReal		sigma0;
	GCLReal		d;
	GCLReal		n_r;
	GCLReal		n_i;
	Colour		rdClr;

	GCLReal		lambda;
	GCLReal		epsilon;
};



#endif
