/*
	File:			Geometry.h

	Function:		General geometrical definitions for points, vectors, etc.
					
	Author(s):		Andrew Willmott

	Copyright:		(c) 1995-2000, Andrew Willmott
 */

#ifndef __Geometry__
#define __Geometry__

#include "GCLConfig.h"

#include "cl/Array.h"
#include "NArray.h"


// --- Fundamental primitives -------------------------------------------------

#ifdef GCL_FLOAT
#include "vl/VLfd.h"
typedef Float	GCLReal;
typedef Vec3f	Point;		// We try and make provision here for points and
typedef Vec3f	Vector;		// vectors being separate types in future...
typedef Vec4f	Vector4;		
typedef Vec4d	Quaternion;
typedef Vec2f	Coord;			

typedef Vec4f	HPoint;
typedef Vec4f	HVector;
typedef Vecf	GCLVec;
#else
#include "vl/VLd.h"
typedef Double	GCLReal;
typedef Vec3d	Point;		
typedef Vec3d	Vector;		
typedef Vec4d	Vector4;		
typedef Vec4d	Quaternion;
typedef Vec2d	Coord;			

typedef Vec4d	HPoint;
typedef Vec4d	HVector;
typedef Vecd	GCLVec;
#endif

typedef Vec3d	MVector;
typedef Vec4d	MVector4;
typedef Mat4d	Transform;
typedef Mat3d	VecTrans;
typedef Mat2d	CoordTrans;
typedef Matd	GCLMat;

typedef Int		Index;

inline Transform Rotation(const Vector &axis, GCLReal theta)
{ Transform result; result.MakeHRot(axis, theta); return(result); }
inline Transform Rotation(const Quaternion &q)
{ Transform result; result.MakeHRot(q); return(result); }
inline Transform Scale(const Vector &s)
{ Transform result; result.MakeHScale(MVector(s[0], s[1], s[2])); return(result); }
inline Transform Scalef(GCLReal r)
{ Transform result; result.MakeHScale(MVector(r, r, r)); return(result); }
inline Transform Shift(const Vector &t)
{ Transform result; result.MakeHTrans(MVector(t[0], t[1], t[2])); return(result); }

inline Transform Scale(GCLReal sx, GCLReal sy, GCLReal sz)
{ Transform result; result.MakeHScale(MVector(sx, sy, sz)); return(result); }
inline Transform Scale(GCLReal s)
{ Transform result; result.MakeHScale(MVector(s, s, s)); return(result); }
inline Transform Shift(GCLReal tx, GCLReal ty, GCLReal tz)
{ Transform result; result.MakeHTrans(MVector(tx, ty, tz)); return(result); }

Transform Align(const Vector &xAxis, const Vector &yAxis, const Vector &zAxis);
Transform AlignToDir(const Vector &dir);


typedef Array<Int>		IndexList;
typedef Array<GCLReal>	ScalarList;
typedef NArray<Point>	PointList;
typedef NArray<Vector>	VectorList;
typedef NArray<Coord>	CoordList;
typedef VectorList 		NormalList;

inline GCLReal DegsToRads(GCLReal degrees)	///< Degrees to radians conversion...
{ return(degrees * (vl_pi / 180.0)); }
inline GCLReal RadsToDegs(GCLReal rads)		///< Radians to degrees conversion...
{ return(rads * (180.0 / vl_pi)); }

IndexList Indexes(Int first, ...);	///< Constructor for index lists
const Index IDX_END = -1;			///< E.g., Indexes(1, 2, 3, 4, 5, IDX_END);

#endif
